"use strict";(self.webpackChunkschedula_form=self.webpackChunkschedula_form||[]).push([[623],{2623:function(e,t,u){u.r(t),u.d(t,{default:function(){return n.Z},getTableBodyUtilityClass:function(){return s.j},tableBodyClasses:function(){return s.Z}});var n=u(1524),s=u(9577)}}]);