"use strict";(self.webpackChunkschedula_form=self.webpackChunkschedula_form||[]).push([[966],{6966:function(e,t,u){u.r(t),u.d(t,{default:function(){return n.Z},getTableUtilityClass:function(){return s.K},tableClasses:function(){return s.Z}});var n=u(7017),s=u(6453)}}]);