"use strict";(self.webpackChunkschedula_form=self.webpackChunkschedula_form||[]).push([[904],{4904:function(e,t,u){u.r(t),u.d(t,{default:function(){return n.Z},getTableHeadUtilityClass:function(){return s.s},tableHeadClasses:function(){return s.Z}});var n=u(1215),s=u(9259)}}]);